*! Version & 17december2007
************************************************************************************************************
* Stata program : pcm
* Estimate the parameters of the Partial Credit Model
* Version 1 : December 17, 2007
*
*
* Jean-benoit Hardouin, Regional Health Observatory of Orlans - France
* jean-benoit.hardouin@neuf.fr
*
* News about this program :http://www.anaqol.org
*
* All the necessary programs on the FreeIRT Project website : http://www.freeirt.org
*
* Copyright 2007 Jean-Benoit Hardouin
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
************************************************************************************************************


program define pcm,eclass
version 8.0
syntax varlist(min=3 numeric) [if] [in] [,rsm fixed(string) short]
preserve
tempfile pcmfile
qui save `pcmfile',replace

/*******************************************************************************
ESTIMATION OF THE PARAMETERS
********************************************************************************/

marksample touse
qui keep if `touse'
qui count
local N=r(N)
tokenize `varlist'
local nbitems : word count `varlist'

if "`rsm'"=="" {
   di in gr "Model: " in ye "Partial Credit Model"
}
else {
   di in gr "Model: " in ye "Rating Scale Model"
}

tempname one var w id item it obs x chosen d score
qui gen `one'=1
qui gen `id'=_n
local modamax=0
forvalues i=1/`nbitems' {
   qui rename ``i'' `var'`i'
   qui su `var'`i'
   local moda`i'=`r(max)'
   if `modamax'<`r(max)' {
      local modamax=r(max)
   }
}
qui genscore `var'1-`var'`nbitems' ,score(`score')
qui collapse (sum) `w'=`one',by(`var'1-`var'`nbitems')
qui gen `id'=_n
qui reshape long `var',i(`id') j(`item')
qui drop if `var'==.
qui gen `obs'=_n
qui expand `=`modamax'+1'
qui sort `id' `item' `obs'
by `obs', sort: gen `x'=_n-1

qui gen `chosen'=`var'==`x'
qui tab `item', gen(`it')
forvalues i=1/`nbitems' {
   forvalues g=1/`modamax' {
      qui gen `d'`i'_`g'=-1*`it'`i'*(`x'>=`g')
   }
}
*di "x d1_1-d5_3 id obs chosen w "
*list `x' `d'1_1-`d'`nbitems'_`modamax' `id' `obs' `chosen' `w' in 1/15 ,compress
qui rename `w' `w'2
bysort `id':egen score=sum(`x'*`chosen')
qui su score
local maxscore=r(max)
*bysort score:su `w'

if "`rsm'"=="" {

   if "`fixed'"!="" {
      *set trace on
      gen offset=0
      forvalues i=1/`nbitems' {
         forvalues mi=1/`moda`i'' {
            replace offset=`fixed'[`i',`mi']*`d'`i'_`mi' if `d'`i'_`mi'!=0
         }
      }
      eq slope:`x'
      qui gllamm `x'  ,offset(offset) i(`id') eqs(slope) link(mlogit) expand(`obs' `chosen' o) weight(`w') adapt trace nocons
   }
   else if "`short'"!="" {
      eq slope:`x'
      qui gllamm `x' `d'1_1-`d'`nbitems'_`modamax',i(`id') eqs(slope) link(mlogit) expand(`obs' `chosen' o) weight(`w') adapt trace nocons init
      tempname bsave Vsave
      matrix `bsave'=e(b)
      matrix `Vsave'=e(V)
      restore
      qui pcm `varlist' , fixed(`bsave')
   }
   else {
      di "no short"
      eq slope:`x'
      gllamm `x' `d'1_1-`d'`nbitems'_`modamax',i(`id') eqs(slope) link(mlogit) expand(`obs' `chosen' o) weight(`w') adapt trace nocons
      *gllapred xb,xb
      *gllapred linpred,linpred
	  *gen latent=linpred-xb
	  *su latent
/*      bysort `id':egen score=sum(`x'*`chosen')
      forvalues i=0/`maxscore' {
         qui su ltm1 if score==`i'
         local lt`i'=r(mean)
         local nblt`i'=r(N)
         qui su lts1 if score==`i'
         local ltse`i'=r(mean)
         di "score `i' : `lt`i'' (`v')"
      }
*/
   }
}
else {
   tempname step n
   forvalues i=2/`modamax' {
      qui gen `step'`i'=-1*(`x'>=`i')
   }
   forvalues i=1/`nbitems' {
      qui gen `n'`var'`i'=(-1)*(`it'`i')*(`x')
   }
   qui sort `id' `item' `x'
   eq slope:`x'
di "x `var' nvar1-nvar`nbitems' step2-step`modamax'"
list `x'  `n'`var'1-`n'`var'`nbitems' `step'2-`step'`modamax' `id' `obs' `chosen' `w' in 1/100 ,compress
*su
   gllamm `x' `n'`var'1-`n'`var'`nbitems' `step'2-`step'`modamax', i(`id') eqs(slope) link(mlogit) expand(`obs' `chosen' o) weight(`w') adapt trace nocons
   *gllapred lt,u
}

tempname b V chol
matrix b=e(b)
matrix V=e(V)
local ll=e(ll)
matrix chol=e(chol)

if "`rsm'"=="" {
   di
   di in gr "Number of observations: " in ye `N'
   di in gr "Number of items: " in ye `nbitems'
   di in gr "Number of parameters: " in  ye `=`nbitems'*`modamax'+1'
   di in gr "Log-likelihood: " in ye `ll'
   di
   di
   di in gr "{hline 100}"
   di in gr "Item" _col(10) "Modality" _col(20) "Parameter" _col(30) "Std Error"
   di in gr "{hline 100}"
   if "`fixed'"=="" {
      forvalues i=1/`nbitems' {
         forvalues j=1/`modamax' {
            if `j'==1 {
               di in ye "``i''" _cont
            }
            local k=(`i'-1)*`modamax'+`j'
            if "`short'"!="" {
               di in ye _col(17) `j' _col(20) %9.6f `bsave'[1,`k'] in ye _col(30) %9.6f (`Vsave'[`k',`k'])^.5
            }
            else {
               di in ye _col(17) `j' _col(20) %9.6f b[1,`k'] in ye _col(30) %9.6f (V[`k',`k'])^.5
            }
         }
         di in gr "{dup 100:-}"
      }
   }
   else {
      forvalues i=1/`nbitems' {
         forvalues j=1/`modamax' {
            if `j'==1 {
               di in ye "``i''" _cont
            }
            local k=(`i'-1)*`modamax'+`j'
            di in ye _col(17) `j' _col(20) %9.6f `fixed'[1,`k'] in ye _col(30) %9.6f (V[`k',`k'])^.5
         }
         di in gr "{dup 100:-}"
      }
   }
   local k=`nbitems'*`modamax'+1
   if "`short'"!="" {
      di in ye "Sigma" in ye _col(20) %9.6f b[1,1] _col(29) %10.6f  (V[1,1])^.5
      di in ye "Variance" in ye _col(20) %9.6f b[1,1]^2  _col(29) %10.6f  2*(V[1,1])^.5*b[1,1]
   }
   else {
      di in ye "Sigma" in ye _col(20) %9.6f b[1,`k'] _col(29) %10.6f  (V[`k',`k'])^.5
      di in ye "Variance" in ye _col(20) %9.6f b[1,`k']^2  _col(29) %10.6f  2*(V[`k',`k'])^.5*b[1,`k']
   }
   di in gr "{hline 100}"
   di
   di
   /*
   di in gr "{hline 100}"
   di in gr "Score" _col(10) "freq." _col(20) "Parameter" _col(30) "Std Error"
   di in gr "{hline 100}"
*set trace on
   forvalues i=0/`maxscore' {
      di in ye `i' _col(10) %8.0f `nblt`i'' _col(20) %9.6f `lt`i'' _col(30) %9.6f `ltse`i''
   }
   */
}



**********************************************





*tempname w pattern item score d negd var
*contract `varlist',freq(`w')
*gen `pattern'=_n
*forvalues i=1/`nbitems' {
*   rename ``i'' `var'`i'
*}
*reshape long `var',i(`pattern') j(`item')
*rename `var' `score'
*tab `item',gen(`d')
*forvalues i=1/`nbitems' {
*   gen `negd'`i'=-`d'`i'
*}

*gllamm `score' `negd'1-`negd'`nbitems',i(`pattern') link(logit) family(binom) weight(`w') nip(12) nocons adapt trace







*restore,not
*drop _all
*qui use `pcmfile'
end
